"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var automerge_1 = __importDefault(require("automerge"));
var updateCodeMirrorDocs_1 = __importDefault(require("./updateCodeMirrorDocs"));
var Mutex_1 = __importDefault(require("./Mutex"));
var updateAutomergeText_1 = __importDefault(require("./updateAutomergeText"));
/**
 * Connect an Automerge document
 *
 * @param notify - a callback that gets called when the doc is updated as the result of an editor change
 * @return ConnectCodeMirror - a function for connecting an Automerge.Text object in the document to a CodeMirror instance
 */
var AutomergeCodeMirror = /** @class */ (function () {
    function AutomergeCodeMirror(change) {
        this.change = change;
        this.mutex = new Mutex_1.default();
        this.codeMirrorByTextId = new Map();
    }
    AutomergeCodeMirror.prototype.getCodeMirror = function (textObjectId) {
        return this.codeMirrorByTextId.get(textObjectId);
    };
    /**
     * Connects a CodeMirror instance to an Automerge.Text object.
     *
     * @param doc the doc with the text
     * @param codeMirror the editor instance
     * @param getText a function that looks up the Automerge.Text object to connect the CodeMirror editor to
     */
    AutomergeCodeMirror.prototype.connectCodeMirror = function (doc, codeMirror, getText) {
        var _this = this;
        var text = getText(doc);
        if (!text) {
            throw new Error("Cannot connect CodeMirror. Did not find text in " + JSON.stringify(doc));
        }
        // Establish the association between the Automerge.Text objectId and the CodeMirror instance.
        // This association is used during the processing of diffs between two Automerge document changes,
        // so the right CodeMirror instance can be found for each Automerge.Text change.
        var textId = automerge_1.default.getObjectId(text);
        this.codeMirrorByTextId.set(textId, codeMirror);
        codeMirror.setValue(text.toString());
        var codeMirrorChangeHandler = function (editor, editorChange) {
            if (editorChange.origin !== 'automerge') {
                _this.mutex.lock();
                _this.change(function (proxy) { return updateAutomergeText_1.default(proxy, getText, editor.getDoc(), editorChange); });
                _this.mutex.release();
            }
        };
        codeMirror.on('change', codeMirrorChangeHandler);
        return function () {
            codeMirror.off('change', codeMirrorChangeHandler);
            _this.codeMirrorByTextId.delete(textId);
        };
    };
    AutomergeCodeMirror.prototype.updateCodeMirrors = function (oldDoc, newDoc) {
        return updateCodeMirrorDocs_1.default(oldDoc, newDoc, this.getCodeMirror.bind(this), this.mutex);
    };
    return AutomergeCodeMirror;
}());
exports.default = AutomergeCodeMirror;
//# sourceMappingURL=AutomergeCodeMirror.js.map